#!/bin/sh
#
# vim: set ts=4
# script to perform the update
#set -x
RUN_FILES_DIR=.

FILES_FILE=files.txt

# Backup the specified file
# If $1 == -m then move it, otherwise copy
backup () {
	if [ "x-m" == "x$1" ]
	then
		mv $1 ${1}.bak
	else
		cp -p $1 ${1}.bak
	fi
	return 0
}

# Install the specified file
# $1 is the file to install
# $2 is the destination (file or dir)
# $3 [optional] chmod argument for installed file
#
# Notes
# - This funciton will not create any required dirs in the install path.
# - It currently does not handle installing anything but single files.
install () {
	local src=$1
	local dest=$2

	# Make the Path to the file
	mkdir -p $( dirname $dest )

	[ -d $dest ] && dest=$dest/$(basename $src)
	
    (cmp -s $src $dest && echo Skipping $src) || (cp -p $src $dest && echo Installing $src to $dest)
	if [ "x" != "x$3" ]
	then
		chmod $3 $dest
	fi
	return 0
}

# Make a symlink to a file
# $1 is the link target
# $2 is the link name
mklink() {
	local lname=$2
	local ltarget=$1
	echo linking $lname to $ltarget
        local testlink=$(readlink ${lname})
        if [ "x${testlink}" != "x${ltarget}" ]
        then
            rm ${lname}
	    ln -s ${ltarget} ${lname}
        else
            echo "Skipping ${ltarget} (link exists)"
        fi
}

# Create file /tmp/reboot_required to indicate to outside application
# that an reboot is needed
require_reboot () {
	touch /tmp/reboot_required
}

# Run any pre-update scripts
# $1 is the prefix
# File names must be <prefix>NN<suffix
# ie. S00dostuff
process_ordered() {
	for ff in ${1}[0-9][0-9]*
	do
		# Check if the script is there.
		[ ! -x $ff ] && continue
		echo Executing $ff
		./$ff || echo $ff Failed
	done
}
#
# process a list of files to install
# $1 is the name of the files list file
process_filelist() {
local fname=$1
local count=0
local line=0
if [ -f ${fname} ]
then
	#loop through the files file, installing the files
	while read inputline;
	do
        line=$(expr $line + 1)
        [ -z "${inputline}"  ] && continue
        [ ${inputline:0:1} == '#' ] && continue
		let fields=$(echo ${inputline}|awk -F, '{print NF}')
		if [ 2 -gt ${fields} ] || [ 3 -lt ${fields} ]
		then
			echo ${fname}:${line}:Bad input line:${inputline}
			continue
		fi
		count=$(expr $count + 1)
		src=$(eval echo "$inputline" | cut -d, -f1)
		dst=$(eval echo "$inputline" | cut -d, -f2)
		mode=$(eval echo "$inputline" | cut -d, -f3)
                if [ "xlink" == "x$mode" ]
                then
			mklink "$src" "$dst"
		elif [ "xrm" == "x$mode" ]
		then
			rm -f "$dst"
		else
			install "$src" "$dst" "$mode"
		fi
	done < ${fname}
	echo ${fname}: $count files processed in ${line} lines
fi

}

####
####    ####    ####    START OF SCRIPT    ####    ####    ####
####
date
echo Running $0

# Process "S" (Start) files
process_ordered S
process_filelist $FILES_FILE
# Process "F" (Finish) files
process_ordered F

echo Update complete...
exit 0
